@@ -4,6 +4,7 @@ try:
     from _thread import get_ident
 except ImportError:
     from thread import get_ident
+from threading import enumerate as enumerate_threads
 try:
     from urllib import quote
 except ImportError:
@@ -32,25 +33,27 @@ class SQLiteConnection(DBAPI):
     schemes = [dbName]
 
     def __init__(self, filename, autoCommit=1, **kw):
-        drivers = kw.pop('driver', None) or 'pysqlite2,sqlite3,sqlite'
+        drivers = kw.pop('driver', None) or \
+            'pysqlite2,sqlite3,sqlite'
         for driver in drivers.split(','):
             driver = driver.strip()
             if not driver:
                 continue
             try:
                 if driver in ('sqlite2', 'pysqlite2'):
-                        from pysqlite2 import dbapi2 as sqlite
-                        self.using_sqlite2 = True
+                    from pysqlite2 import dbapi2 as sqlite
+                    self.using_sqlite2 = True
                 elif driver == 'sqlite3':
-                        import sqlite3 as sqlite
-                        self.using_sqlite2 = True
+                    import sqlite3 as sqlite
+                    self.using_sqlite2 = True
                 elif driver in ('sqlite', 'sqlite1'):
-                        import sqlite
-                        self.using_sqlite2 = False
+                    import sqlite
+                    self.using_sqlite2 = False
                 else:
                     raise ValueError(
                         'Unknown SQLite driver "%s", '
-                        'expected pysqlite2, sqlite3 or sqlite' % driver)
+                        'expected pysqlite2, sqlite3 '
+                        'or sqlite' % driver)
             except ImportError:
                 pass
             else:
@@ -59,8 +62,8 @@ class SQLiteConnection(DBAPI):
             raise ImportError(
                 'Cannot find an SQLite driver, tried %s' % drivers)
         if self.using_sqlite2:
-            sqlite.encode = base64.encodestring
-            sqlite.decode = base64.decodestring
+            sqlite.encode = base64.b64encode
+            sqlite.decode = base64.b64decode
         self.module = sqlite
         self.filename = filename  # full path to sqlite-db-file
         self._memory = filename == ':memory:'
@@ -145,6 +148,7 @@ class SQLiteConnection(DBAPI):
             self._connectionNumbers[id(conn)] = self._connectionCount
             self._connectionCount += 1
             return conn
+        self._releaseUnusedConnections()
         threadid = get_ident()
         if (self._pool is not None and threadid in self._threadPool):
             conn = self._threadPool[threadid]
@@ -170,12 +174,15 @@ class SQLiteConnection(DBAPI):
             return
         threadid = self._threadOrigination.get(id(conn))
         DBAPI.releaseConnection(self, conn, explicit=explicit)
-        if (self._pool is not None and threadid and
-                threadid not in self._threadPool):
+        if (self._pool is not None and threadid
+                and threadid not in self._threadPool):
             self._threadPool[threadid] = conn
         else:
             if self._pool and conn in self._pool:
                 self._pool.remove(conn)
+            if threadid:
+                del self._threadOrigination[id(conn)]
+                del self._threadPool[threadid]
             conn.close()
 
     def _setAutoCommit(self, conn, auto):
@@ -206,9 +213,23 @@ class SQLiteConnection(DBAPI):
         conn.text_factory = str  # Convert text data to str, not unicode
         return conn
 
+    def _releaseUnusedConnections(self):
+        """Release connections from threads that're no longer active"""
+        thread_ids = set(t.ident for t in enumerate_threads())
+        tp_set = set(self._threadPool)
+        unused_connections = [
+            self._threadPool[tid] for tid in tp_set - thread_ids
+        ]
+        for unused_connection in unused_connections:
+            try:
+                self.releaseConnection(unused_connection, explicit=True)
+            except self.module.ProgrammingError:
+                pass  # Ignore error in `conn.close()` from a different thread
+
     def close(self):
         DBAPI.close(self)
         self._threadPool = {}
+        self._threadOrigination = {}
         if self._memory:
             self._memoryConn.close()
             self.makeMemoryConnection()
@@ -258,6 +279,7 @@ class SQLiteConnection(DBAPI):
         # lastrowid is a DB-API extension from "PEP 0249":
         if id is None:
             id = int(c.lastrowid)
+        c.close()
         if self.debugOutput:
             self.printDebug(conn, id, 'QueryIns', 'result')
         return id
